@@ -241,7 +241,11 @@ class ProjectBundle(BaseModel):
                     )
 
         # Load artifacts in parallel using ThreadPoolExecutor
-        max_workers = min(os.cpu_count() or 4, 8, len(load_tasks))  # Cap at 8 workers
+        # In test mode, use fewer workers to avoid resource contention
+        if os.environ.get("TEST_MODE") == "true":
+            max_workers = max(1, min(2, len(load_tasks)))  # Max 2 workers in test mode
+        else:
+            max_workers = min(os.cpu_count() or 4, 8, len(load_tasks))  # Cap at 8 workers
         completed_count = current
 
         def load_artifact(artifact_name: str, artifact_path: Path, validator: Callable) -> tuple[str, Any]:
@@ -253,6 +257,8 @@ class ProjectBundle(BaseModel):
         if load_tasks:
             executor = ThreadPoolExecutor(max_workers=max_workers)
             interrupted = False
+            # In test mode, use wait=False to avoid hanging on shutdown
+            wait_on_shutdown = os.environ.get("TEST_MODE") != "true"
             try:
                 # Submit all tasks
                 future_to_task = {
@@ -308,7 +314,7 @@ class ProjectBundle(BaseModel):
                 raise
             finally:
                 if not interrupted:
-                    executor.shutdown(wait=True)
+                    executor.shutdown(wait=wait_on_shutdown)
                 else:
                     executor.shutdown(wait=False)
 
@@ -395,7 +401,11 @@ class ProjectBundle(BaseModel):
             save_tasks.append((f"features/{feature_file}", feature_path, feature.model_dump()))
 
         # Save artifacts in parallel using ThreadPoolExecutor
-        max_workers = min(os.cpu_count() or 4, 8, len(save_tasks))  # Cap at 8 workers
+        # In test mode, use fewer workers to avoid resource contention
+        if os.environ.get("TEST_MODE") == "true":
+            max_workers = max(1, min(2, len(save_tasks)))  # Max 2 workers in test mode
+        else:
+            max_workers = min(os.cpu_count() or 4, 8, len(save_tasks))  # Cap at 8 workers
         completed_count = 0
         checksums: dict[str, str] = {}  # Track checksums for manifest update
         feature_indices: list[FeatureIndex] = []  # Track feature indices
@@ -410,6 +420,8 @@ class ProjectBundle(BaseModel):
         if save_tasks:
             executor = ThreadPoolExecutor(max_workers=max_workers)
             interrupted = False
+            # In test mode, use wait=False to avoid hanging on shutdown
+            wait_on_shutdown = os.environ.get("TEST_MODE") != "true"
             try:
                 # Submit all tasks
                 future_to_task = {
@@ -472,7 +484,7 @@ class ProjectBundle(BaseModel):
                 raise
             finally:
                 if not interrupted:
-                    executor.shutdown(wait=True)
+                    executor.shutdown(wait=wait_on_shutdown)
                 else:
                     executor.shutdown(wait=False)
 
@@ -555,12 +567,16 @@ class ProjectBundle(BaseModel):
         content_hash = None
         if include_hash:
             # Compute hash of all aspects combined
+            # NOTE: Exclude clarifications from hash - they are review metadata, not plan content
+            # This ensures hash stability across review sessions (clarifications change but plan doesn't)
+            # IMPORTANT: Sort features by key to ensure deterministic hash regardless of dict insertion order
+            sorted_features = sorted(self.features.items(), key=lambda x: x[0])
             bundle_dict = {
                 "idea": self.idea.model_dump() if self.idea else None,
                 "business": self.business.model_dump() if self.business else None,
                 "product": self.product.model_dump(),
-                "features": [f.model_dump() for f in self.features.values()],
-                "clarifications": self.clarifications.model_dump() if self.clarifications else None,
+                "features": [f.model_dump() for _, f in sorted_features],
+                # Exclude clarifications - they are review metadata, not part of the plan content
             }
             bundle_json = json.dumps(bundle_dict, sort_keys=True, default=str)
             content_hash = hashlib.sha256(bundle_json.encode("utf-8")).hexdigest()
