@@ -157,6 +157,12 @@ class BridgeSync:
                 errors.append(f"Artifact not found: {artifact_path}")
                 return SyncResult(success=False, operations=operations, errors=errors, warnings=warnings)
 
+            # Conflict detection: warn that bundle will be updated
+            warnings.append(
+                f"Importing {artifact_key} from {artifact_path}. "
+                "This will update the project bundle. Existing bundle content may be modified."
+            )
+
             # Load project bundle
             from specfact_cli.utils.structure import SpecFactStructure
 
@@ -254,8 +260,8 @@ class BridgeSync:
             project_bundle: Project bundle to update
         """
         # Basic markdown import (placeholder for future implementation)
-        content = artifact_path.read_text(encoding="utf-8")
-        # Parse and update bundle as needed
+        # TODO: Parse markdown content and update bundle
+        _ = artifact_path.read_text(encoding="utf-8")  # Placeholder for future parsing
 
     @beartype
     @require(lambda bundle_name: isinstance(bundle_name, str) and len(bundle_name) > 0, "Bundle name must be non-empty")
@@ -302,6 +308,13 @@ class BridgeSync:
             # Resolve artifact path
             artifact_path = self.resolve_artifact_path(artifact_key, feature_id, bundle_name)
 
+            # Conflict detection: warn if file exists (will be overwritten)
+            if artifact_path.exists():
+                warnings.append(
+                    f"Target file already exists: {artifact_path}. "
+                    "Will overwrite with bundle content. Use --overwrite flag to suppress this warning."
+                )
+
             # Ensure parent directory exists
             artifact_path.parent.mkdir(parents=True, exist_ok=True)
 
@@ -456,9 +469,9 @@ class BridgeSync:
         # Sync each feature
         for feature_id in feature_ids:
             # Import from tool → bundle
-            for artifact_key in ["specification", "plan", "tasks"]:
-                if artifact_key in self.bridge_config.artifacts:
-                    import_result = self.import_artifact(artifact_key, feature_id, bundle_name)
+            for _artifact_key in ["specification", "plan", "tasks"]:
+                if _artifact_key in self.bridge_config.artifacts:
+                    import_result = self.import_artifact(_artifact_key, feature_id, bundle_name)
                     operations.extend(import_result.operations)
                     errors.extend(import_result.errors)
                     warnings.extend(import_result.warnings)
@@ -505,4 +518,3 @@ class BridgeSync:
                                 feature_ids.append(item.name)
 
         return feature_ids
-
